#include "volumemanager.h"

VolumeManager::Pointer VolumeManager::mInstance;

VolumeManager::VolumeManager() : 
    m_StateMachine(this),
    mVolumeIdToBeRetrieved(-1)
{
    // States
    igstkAddStateMacro(Initial);
    igstkAddStateMacro(AttemptingToLoadDicomVolume);
    igstkAddStateMacro(DicomVolumeReady);

    // Inputs
    igstkAddInputMacro(LoadDicomVolume);
    igstkAddInputMacro(DicomVolumeLoadingSuccess);
    igstkAddInputMacro(DicomVolumeLoadingFailure);
    igstkAddInputMacro(GetDicomVolume);
    igstkAddInputMacro(InvalidDicomVolume);

    // Transitions
    igstkAddTransitionMacro(Initial, LoadDicomVolume, 
                            AttemptingToLoadDicomVolume, loadImage);
    igstkAddTransitionMacro(AttemptingToLoadDicomVolume, DicomVolumeLoadingSuccess, DicomVolumeReady, no);
    igstkAddTransitionMacro(AttemptingToLoadDicomVolume, DicomVolumeLoadingFailure, Initial, no);
    igstkAddTransitionMacro(DicomVolumeReady, GetDicomVolume, DicomVolumeReady, getDicomVolume);
    igstkAddTransitionMacro(DicomVolumeReady, InvalidDicomVolume, DicomVolumeReady, no);

    // Statemachine
    igstkSetInitialStateMacro(Initial);
    m_StateMachine.SetReadyToRun();
}

VolumeManager::~VolumeManager()
{}

void VolumeManager::requestLoadDicomVolume( const std::string & path)
{
    mDicomVolumePath = path;
    m_StateMachine.PushInput(m_LoadDicomVolumeInput);
    m_StateMachine.ProcessInputs();
}

void VolumeManager::requestGetVolume(int volumeId)
{
    if( (volumeId >= 0) && ( mSpatialObjects.size()) )
    {
        mVolumeIdToBeRetrieved = volumeId;
        m_StateMachine.PushInput(m_GetDicomVolumeInput) ;
    }
    else
    {
        m_StateMachine.PushInput(m_InvalidDicomVolumeInput);
    }

    m_StateMachine.ProcessInputs();
}

void VolumeManager::noProcessing()
{}

// Loads a DICOM image and adds it to the internal vector of volumes
void VolumeManager::loadImageProcessing()
{
    std::cout << "loadImageProcessing()\n";
    typedef igstk::MRImageReader ReaderType;
    ReaderType::Pointer imageReader = ReaderType::New();
    
    MRImageObserver::Pointer imageReadyObserver = MRImageObserver::New();
    imageReader->AddObserver(igstk::MRImageReader::ImageModifiedEvent(), imageReadyObserver);

    // LUIS NOTE: Undefined type:  ObserveInvalidRequestErrorEvent(imageReader);
    // LUIS NOTE: Undefined type:  ObserveImageReadingErrorEvent(imageReader);

//    ReaderType::DirectoryNameType directoryName = mDicomVolumePath;
    imageReader->RequestSetDirectory(mDicomVolumePath);
    imageReader->RequestReadImage();
    imageReader->RequestGetImage();
    if(imageReadyObserver->GotMRImage())
    {
        igstk::MRImageSpatialObject::Pointer  theImage = imageReadyObserver->GetMRImage();
        mSpatialObjects.push_back(theImage);

        ImageReadyEvent event;
        event.Set(mSpatialObjects.size() - 1);
        InvokeEvent(event);
        m_StateMachine.PushInput(m_DicomVolumeLoadingSuccessInput);
        m_StateMachine.ProcessInputs();
    }
    else
    {
        m_StateMachine.PushInput(m_DicomVolumeLoadingFailureInput);
        m_StateMachine.ProcessInputs();
    }
}

void VolumeManager::getDicomVolumeProcessing()
{
    FoundVolumeEvent event;
    event.Set(mSpatialObjects.at(mVolumeIdToBeRetrieved));
    InvokeEvent(event);
}

VolumeManager::Pointer VolumeManager::instance()
{
  std::cout << "VolumeManager::instance()" << std::endl;
  if( mInstance.IsNull() )
    {
    std::cout << "VolumeManager::New()" << std::endl;
    mInstance = VolumeManager::New();
    }
  std::cout << "mInstance = " << mInstance << std::endl;
  return mInstance;
}
