#ifndef VOLUMEMANAGER_H_
#define VOLUMEMANAGER_H_

#include <vector>
#include <igstkSpatialObject.h>
#include <igstkStateMachine.h>
#include <igstkMRImageReader.h>
#include <igstkDICOMImageReader.h>
#include <igstkMRImageSpatialObject.h>
#include <igstkEvents.h>

class VolumeManager : public igstk::Object
{
public:
    igstkStandardClassBasicTraitsMacro(VolumeManager, igstk::Object);
    igstkStateMachineMacro();

    static Pointer instance(); 
    
    void requestLoadDicomVolume( const std::string & path);
    void requestGetVolume(int volumeId);

    igstkLoadedEventMacro(ImageReadyEvent, igstk::IGSTKEvent, igstk::EventHelperType::SignedIntType);
    igstkLoadedEventMacro(DicomImageFailedLoading, igstk::IGSTKEvent, igstk::EventHelperType::StringType);
    igstkLoadedObjectEventMacro(FoundVolumeEvent, igstk::IGSTKEvent, igstk::MRImageSpatialObject);
private:
    VolumeManager();
    ~VolumeManager();
    igstkNewMacro(Self);
    int addVolume(igstk::SpatialObject::Pointer);
    static Pointer mInstance;
    std::vector<igstk::MRImageSpatialObject::Pointer> mSpatialObjects;

    // States
    igstkDeclareStateMacro(Initial);
    igstkDeclareStateMacro(AttemptingToLoadDicomVolume);
    igstkDeclareStateMacro(DicomVolumeReady);
 
    // Inputs
    igstkDeclareInputMacro(LoadDicomVolume);
    igstkDeclareInputMacro(DicomVolumeLoadingSuccess);
    igstkDeclareInputMacro(DicomVolumeLoadingFailure);
    igstkDeclareInputMacro(GetDicomVolume);
    igstkDeclareInputMacro(InvalidDicomVolume);
    
    // Action methods
    void noProcessing();
    void loadImageProcessing();
    void getDicomVolumeProcessing();

    std::string mDicomVolumePath;
    int mVolumeIdToBeRetrieved;
    typedef igstk::DICOMInvalidRequestErrorEvent InvalidRequestErrorEvent;
    typedef igstk::DICOMImageReadingErrorEvent ImageReadingErrorEvent;
    igstkEventTransductionMacro(DICOMInvalidRequestErrorEvent, DicomVolumeLoadingFailureInput);
    igstkEventTransductionMacro(DICOMImageReadingErrorEvent, DicomVolumeLoadingFailureInput);

    igstkObserverObjectMacro(MRImage, igstk::MRImageReader::ImageModifiedEvent, igstk::MRImageSpatialObject);
};
#endif
